"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
const react_component_renderer_1 = require("../react-component-renderer");
const react_component_render_helper_1 = require("../react-component-render-helper");
const imports_1 = require("../imports");
const forms_1 = require("../forms");
const form_renderer_helper_1 = require("../forms/form-renderer-helper");
const layout_helpers_1 = require("../utils/forms/layout-helpers");
const value_props_1 = require("../forms/form-renderer-helper/value-props");
const render_checkers_1 = require("../forms/form-renderer-helper/render-checkers");
class FormRenderer extends react_component_renderer_1.ReactComponentRenderer {
    constructor(component, form, // we are passing in form here as it's the top level component
    componentMetadata, importCollection, parent) {
        super(component, componentMetadata, importCollection, parent);
        this.form = form;
        this.componentMetadata = componentMetadata;
        this.importCollection = importCollection;
        this.parent = parent;
    }
    renderElement(renderChildren) {
        var _a;
        const children = (_a = this.component.children) !== null && _a !== void 0 ? _a : [];
        const element = typescript_1.factory.createJsxElement(this.renderCollectionOpeningElement(), renderChildren ? renderChildren(children) : [], typescript_1.factory.createJsxClosingElement(typescript_1.factory.createIdentifier(this.component.componentType)));
        this.importCollection.addImport('@aws-amplify/ui-react', this.component.componentType);
        if (this.form.dataType.dataSourceType === 'DataStore') {
            this.importCollection.addImport('aws-amplify', 'DataStore');
        }
        if ((0, layout_helpers_1.hasTokenReference)(this.componentMetadata)) {
            this.importCollection.addImport(imports_1.ImportSource.UI_REACT, 'useTheme');
        }
        return element;
    }
    renderCollectionOpeningElement() {
        const propsArray = Object.entries(this.component.properties).map(([key, value]) => (0, react_component_render_helper_1.buildOpeningElementProperties)(this.componentMetadata, value, key));
        propsArray.push(...(0, react_component_render_helper_1.buildLayoutProperties)(this.componentMetadata.formMetadata));
        const submitAttribute = this.getFormOnSubmitAttribute();
        propsArray.push(submitAttribute);
        this.addPropsSpreadAttributes(propsArray);
        return typescript_1.factory.createJsxOpeningElement(typescript_1.factory.createIdentifier(this.component.componentType), undefined, typescript_1.factory.createJsxAttributes(propsArray));
    }
    /**
     * generates the necessary function call if using
     * - datastore
     * - custom
     */
    getOnSubmitDSCall() {
        const { dataType: { dataSourceType, dataTypeName }, formActionType, } = this.form;
        const importedModelName = this.importCollection.getMappedAlias(imports_1.ImportSource.LOCAL_MODELS, dataTypeName);
        const { formMetadata } = this.componentMetadata;
        if (!formMetadata) {
            throw new Error(`Form Metadata is missing from form: ${this.component.name}`);
        }
        const hasManyFieldConfigs = (0, form_renderer_helper_1.getHasManyFieldConfigs)(formMetadata.fieldConfigs);
        const onSubmitIdentifier = typescript_1.factory.createIdentifier('onSubmit');
        if (dataSourceType === 'Custom') {
            return [
                typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(onSubmitIdentifier, undefined, [typescript_1.factory.createIdentifier('modelFields')]))),
            ];
        }
        if (dataSourceType === 'DataStore') {
            return [
                typescript_1.factory.createIfStatement(onSubmitIdentifier, typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createCallExpression(onSubmitIdentifier, undefined, [
                        typescript_1.factory.createIdentifier('modelFields'),
                    ]))),
                ], true), undefined),
                typescript_1.factory.createTryStatement(typescript_1.factory.createBlock([
                    ...(0, forms_1.buildDataStoreExpression)(formActionType, importedModelName, hasManyFieldConfigs),
                    // call onSuccess hook if it exists
                    typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('onSuccess'), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('onSuccess'), undefined, [
                            typescript_1.factory.createIdentifier('modelFields'),
                        ])),
                    ], true), undefined),
                    // call the reset function if clearOnSuccess is true
                    ...(0, value_props_1.resetFunctionCheck)({ formActionType, dataSourceType }),
                ], true), typescript_1.factory.createCatchClause(typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('err'), undefined, undefined, undefined), typescript_1.factory.createBlock([
                    typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('onError'), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('onError'), undefined, [
                            typescript_1.factory.createIdentifier('modelFields'),
                            typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('err'), typescript_1.factory.createIdentifier('message')),
                        ])),
                    ], true), undefined),
                ], true)), undefined),
            ];
        }
        throw new Error(`${dataSourceType} is not supported in form:onSubmit`);
    }
    getFormOnSubmitAttribute() {
        const { dataType: { dataSourceType }, } = this.form;
        const { formMetadata } = this.componentMetadata;
        const hasModelField = (formMetadata === null || formMetadata === void 0 ? void 0 : formMetadata.fieldConfigs) && Object.values(formMetadata.fieldConfigs).some((config) => (0, render_checkers_1.isModelDataType)(config));
        return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('onSubmit'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('event'), undefined, typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('SyntheticEvent'), undefined), undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('event'), typescript_1.factory.createIdentifier('preventDefault')), undefined, [])),
            (0, form_renderer_helper_1.buildModelFieldObject)(dataSourceType !== 'DataStore', formMetadata === null || formMetadata === void 0 ? void 0 : formMetadata.fieldConfigs),
            ...(0, form_renderer_helper_1.onSubmitValidationRun)(hasModelField),
            ...this.getOnSubmitDSCall(),
        ], false))));
    }
}
exports.default = FormRenderer;
//# sourceMappingURL=form.js.map