"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getModelsToImport = exports.buildDisplayValueFunction = exports.getDisplayValueObject = exports.getAutocompleteOptionsProp = exports.extractModelAndKey = exports.getDisplayValueObjectName = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const datastore_1 = require("@aws-amplify/datastore");
const typescript_1 = require("typescript");
const react_component_render_helper_1 = require("../../react-component-render-helper");
const form_state_1 = require("./form-state");
const invalid_variable_helpers_1 = require("./invalid-variable-helpers");
const render_checkers_1 = require("./render-checkers");
exports.getDisplayValueObjectName = 'getDisplayValue';
/**
  authorRecords.map((r) => ({
    id: r.id,
    label: r.id,
  }))
 */
/**
  examples:
  for model -
  authorRecords.map((r) => ({
    id: r.id,
    label: getDisplayValue['primaryAuthor']?.(r) ?? r.id,
  }))

  for scalar -
  authorRecords.map((r) => ({
    id: r.id,
    label: r.id,
  }))
 */
function getModelTypeSuggestions({ modelName, fieldName, key, isModelType, }) {
    const recordString = 'r';
    const labelExpression = isModelType
        ? typescript_1.factory.createBinaryExpression(typescript_1.factory.createCallChain((0, invalid_variable_helpers_1.getElementAccessExpression)('getDisplayValue', fieldName), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), undefined, [typescript_1.factory.createIdentifier(recordString)]), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionQuestionToken), (0, invalid_variable_helpers_1.getElementAccessExpression)(recordString, key))
        : (0, invalid_variable_helpers_1.getElementAccessExpression)(recordString, key);
    return typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier((0, form_state_1.getRecordsName)(modelName)), typescript_1.factory.createIdentifier('map')), undefined, [
        typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(recordString), undefined, undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createParenthesizedExpression(typescript_1.factory.createObjectLiteralExpression([
            typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('id'), (0, invalid_variable_helpers_1.getElementAccessExpression)(recordString, key)),
            typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('label'), labelExpression),
        ], true))),
    ]);
}
function extractModelAndKey(valueMappings) {
    let model;
    let key;
    const bindingProperty = (valueMappings === null || valueMappings === void 0 ? void 0 : valueMappings.bindingProperties) && Object.values(valueMappings.bindingProperties)[0];
    if (bindingProperty && bindingProperty.type === 'Data') {
        model = bindingProperty.bindingProperties.model;
        const { value } = valueMappings.values[0];
        if ((0, react_component_render_helper_1.isBoundProperty)(value) && value.bindingProperties.field) {
            key = value.bindingProperties.field;
        }
    }
    return { model, key };
}
exports.extractModelAndKey = extractModelAndKey;
/**
    example:
    options={authorRecords.map(r) => ({
        id: r.id,
        label: getDisplayValue['primaryAuthor']?.(r) ?? r.id,
    }))}
 */
function getAutocompleteOptionsProp({ fieldName, fieldConfig, }) {
    let options;
    const { valueMappings } = fieldConfig;
    const { model, key } = extractModelAndKey(valueMappings);
    if (model && key) {
        options = getModelTypeSuggestions({
            modelName: model,
            fieldName,
            key,
            isModelType: (0, render_checkers_1.isModelDataType)(fieldConfig),
        });
    }
    if (!options) {
        throw new codegen_ui_1.InvalidInputError(`Invalid value mappings on ${fieldName}`);
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('options'), typescript_1.factory.createJsxExpression(undefined, options));
}
exports.getAutocompleteOptionsProp = getAutocompleteOptionsProp;
// impure helper
/* eslint-disable no-param-reassign */
function replaceProperty(prop, toReplace, replaceWith) {
    if ((0, react_component_render_helper_1.isBoundProperty)(prop) && prop.bindingProperties.property === toReplace) {
        prop.bindingProperties.property = replaceWith;
    }
    if ((0, react_component_render_helper_1.isConcatenatedProperty)(prop)) {
        prop.concat.forEach((subProp) => replaceProperty(subProp, toReplace, replaceWith));
    }
}
/* eslint-enable no-param-reassign */
function getDisplayValueObject(displayValueFunctions) {
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(exports.getDisplayValueObjectName), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(displayValueFunctions, true)),
    ], typescript_1.NodeFlags.Const));
}
exports.getDisplayValueObject = getDisplayValueObject;
// example - primaryAuthor: (record) => record?.name,
function buildDisplayValueFunction(fieldName, fieldConfig) {
    const recordString = 'record';
    const propertyName = (0, codegen_ui_1.isValidVariableName)(fieldName)
        ? typescript_1.factory.createIdentifier(fieldName)
        : typescript_1.factory.createStringLiteral(fieldName);
    let additionalStatements = [];
    let renderedDisplayValue = typescript_1.factory.createPropertyAccessChain(typescript_1.factory.createIdentifier(recordString), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('id'));
    if ((0, render_checkers_1.isModelDataType)(fieldConfig) && fieldConfig.valueMappings) {
        const valueConfig = fieldConfig.valueMappings.values[0];
        if (valueConfig) {
            const displayValueProperty = valueConfig.displayValue || valueConfig.value;
            const modelName = fieldConfig.dataType.model;
            replaceProperty(displayValueProperty, modelName, recordString);
            if ((0, react_component_render_helper_1.isConcatenatedProperty)(displayValueProperty)) {
                renderedDisplayValue = (0, react_component_render_helper_1.buildConcatExpression)(displayValueProperty);
            }
            else if ((0, react_component_render_helper_1.isBoundProperty)(displayValueProperty)) {
                renderedDisplayValue = (0, react_component_render_helper_1.buildBindingExpression)(displayValueProperty);
            }
        }
    }
    if ((0, datastore_1.isEnumFieldType)(fieldConfig.dataType) && fieldConfig.valueMappings && fieldConfig.isArray) {
        const displayValueMapName = `enumDisplayValueMap`;
        additionalStatements = [
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(displayValueMapName), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(fieldConfig.valueMappings.values.map((v) => {
                    let value = '';
                    let displayValue = '';
                    if ((0, react_component_render_helper_1.isFixedPropertyWithValue)(v.value)) {
                        value = v.value.value.toString();
                    }
                    if (v.displayValue && (0, react_component_render_helper_1.isFixedPropertyWithValue)(v.displayValue)) {
                        displayValue = v.displayValue.value.toString();
                    }
                    if (value === '') {
                        throw Error('Enum cannot have an empty value');
                    }
                    return typescript_1.factory.createPropertyAssignment(typescript_1.factory.createStringLiteral(value), typescript_1.factory.createStringLiteral(displayValue !== null && displayValue !== void 0 ? displayValue : value));
                }), true)),
            ], typescript_1.NodeFlags.Const)),
        ];
        renderedDisplayValue = typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier(displayValueMapName), typescript_1.factory.createIdentifier(recordString));
    }
    return typescript_1.factory.createPropertyAssignment(propertyName, typescript_1.factory.createArrowFunction(undefined, undefined, [
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(recordString), undefined, undefined, undefined),
    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), additionalStatements.length
        ? typescript_1.factory.createBlock([...additionalStatements, typescript_1.factory.createReturnStatement(renderedDisplayValue)], false)
        : renderedDisplayValue));
}
exports.buildDisplayValueFunction = buildDisplayValueFunction;
function getModelsToImport(fieldConfig) {
    var _a;
    const modelDependencies = [];
    if (fieldConfig.valueMappings && fieldConfig.valueMappings.bindingProperties) {
        Object.values(fieldConfig.valueMappings.bindingProperties).forEach((prop) => {
            if (prop.type === 'Data' && prop.bindingProperties.model) {
                modelDependencies.push(prop.bindingProperties.model);
            }
        });
    }
    // Import join table model
    if (((_a = fieldConfig.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY' && fieldConfig.relationship.relatedJoinTableName) {
        modelDependencies.push(fieldConfig.relationship.relatedJoinTableName);
    }
    return modelDependencies;
}
exports.getModelsToImport = getModelsToImport;
//# sourceMappingURL=display-value.js.map