"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHasManyFieldConfigs = exports.isManyToManyRelationship = exports.mapFromFieldConfigs = void 0;
const display_value_1 = require("./display-value");
const render_checkers_1 = require("./render-checkers");
const validation_1 = require("./validation");
/**
 * This helper returns an evolving collection of artifacts produced by iterating through
 * the fieldConfigs object. Its purpose is to prevent iterating through this object
 * multiple times to shave off compute time.
 */
function mapFromFieldConfigs(fieldConfigs) {
    const validationsForField = [];
    const dataTypesMap = {};
    const displayValueFunctions = [];
    const modelsToImport = [];
    let usesArrayField = false;
    Object.entries(fieldConfigs).forEach(([fieldName, fieldConfig]) => {
        // dataTypesMap
        let dataTypeKey;
        if (fieldConfig.dataType) {
            if (typeof fieldConfig.dataType === 'string') {
                dataTypeKey = fieldConfig.dataType;
            }
        }
        if (dataTypeKey) {
            if (dataTypesMap[dataTypeKey]) {
                dataTypesMap[dataTypeKey].push(fieldName);
            }
            else {
                dataTypesMap[dataTypeKey] = [fieldName];
            }
        }
        // usesArrayField
        if ((0, render_checkers_1.shouldWrapInArrayField)(fieldConfig)) {
            usesArrayField = true;
        }
        // displayValue
        if ((0, render_checkers_1.shouldImplementDisplayValueFunction)(fieldConfig)) {
            displayValueFunctions.push((0, display_value_1.buildDisplayValueFunction)(fieldName, fieldConfig));
        }
        // modelsToImport
        modelsToImport.push(...(0, display_value_1.getModelsToImport)(fieldConfig));
        // validation
        validationsForField.push((0, validation_1.buildValidationForField)(fieldName, fieldConfig.validationRules));
    });
    return {
        validationsObject: (0, validation_1.buildValidations)(validationsForField),
        dataTypesMap,
        displayValueObject: displayValueFunctions.length ? (0, display_value_1.getDisplayValueObject)(displayValueFunctions) : undefined,
        modelsToImport,
        usesArrayField,
    };
}
exports.mapFromFieldConfigs = mapFromFieldConfigs;
const isManyToManyRelationship = (fieldConfigMetaData) => { var _a; return ((_a = fieldConfigMetaData.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY' && fieldConfigMetaData.relationship.relatedJoinTableName; };
exports.isManyToManyRelationship = isManyToManyRelationship;
const getHasManyFieldConfigs = (fieldConfigs) => {
    return Object.entries(fieldConfigs).filter(([, fieldConfigMetaData]) => { var _a; return ((_a = fieldConfigMetaData.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY'; });
};
exports.getHasManyFieldConfigs = getHasManyFieldConfigs;
//# sourceMappingURL=map-from-fieldConfigs.js.map