"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildHasManyRelationshipDataStoreStatements = exports.buildGetRelationshipModels = exports.buildManyToManyRelationshipDataStoreStatements = exports.buildRelationshipQuery = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
const react_studio_template_renderer_helper_1 = require("../../react-studio-template-renderer-helper");
const imports_1 = require("../../imports");
const helpers_1 = require("../../helpers");
const buildRelationshipQuery = (relationship, importCollection) => {
    const { relatedModelName } = relationship;
    const itemsName = (0, form_state_1.getRecordsName)(relatedModelName);
    const objectProperties = [
        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('type'), typescript_1.factory.createStringLiteral('collection')),
        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('model'), typescript_1.factory.createIdentifier(importCollection.getMappedAlias(imports_1.ImportSource.LOCAL_MODELS, relatedModelName))),
    ];
    return (0, react_studio_template_renderer_helper_1.buildBaseCollectionVariableStatement)(itemsName, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('useDataStoreBinding'), undefined, [
        typescript_1.factory.createObjectLiteralExpression(objectProperties, true),
    ]));
};
exports.buildRelationshipQuery = buildRelationshipQuery;
const buildManyToManyRelationshipDataStoreStatements = (dataStoreActionType, modelName, hasManyFieldConfig) => {
    const [fieldName, fieldConfigMetaData] = hasManyFieldConfig;
    const { relatedModelField, relatedJoinFieldName, relatedJoinTableName } = fieldConfigMetaData.relationship;
    if (dataStoreActionType === 'update') {
        const linkedDataName = (0, form_state_1.getLinkedDataName)(fieldName);
        const dataToLinkMap = `${(0, helpers_1.lowerCaseFirst)(fieldName)}ToLinkMap`;
        const dataToUnlinkMap = `${(0, helpers_1.lowerCaseFirst)(fieldName)}ToUnLinkMap`;
        const updatedMap = `${(0, helpers_1.lowerCaseFirst)(fieldName)}Map`;
        const originalMap = `${linkedDataName}Map`;
        return [
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(dataToLinkMap), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Map'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(dataToUnlinkMap), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Map'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(updatedMap), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Map'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(originalMap), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Map'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('count'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(updatedMap), typescript_1.factory.createIdentifier('get')), undefined, [
                            typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                        ])),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('newCount'), undefined, undefined, typescript_1.factory.createConditionalExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.PlusToken), typescript_1.factory.createNumericLiteral('1')), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createNumericLiteral('1'))),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(updatedMap), typescript_1.factory.createIdentifier('set')), undefined, [
                        typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                        typescript_1.factory.createIdentifier('newCount'),
                    ])),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(linkedDataName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('count'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(originalMap), typescript_1.factory.createIdentifier('get')), undefined, [
                            typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                        ])),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('newCount'), undefined, undefined, typescript_1.factory.createConditionalExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.PlusToken), typescript_1.factory.createNumericLiteral('1')), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createNumericLiteral('1'))),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(originalMap), typescript_1.factory.createIdentifier('set')), undefined, [
                        typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                        typescript_1.factory.createIdentifier('newCount'),
                    ])),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(originalMap), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('count'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('id'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('newCount'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(updatedMap), typescript_1.factory.createIdentifier('get')), undefined, [typescript_1.factory.createIdentifier('id')])),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('newCount'), typescript_1.factory.createBlock([
                        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('diffCount'), undefined, undefined, typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.MinusToken), typescript_1.factory.createIdentifier('newCount'))),
                        ], typescript_1.NodeFlags.Const)),
                        typescript_1.factory.createIfStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('diffCount'), typescript_1.factory.createToken(typescript_1.SyntaxKind.GreaterThanToken), typescript_1.factory.createNumericLiteral('0')), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToUnlinkMap), typescript_1.factory.createIdentifier('set')), undefined, [typescript_1.factory.createIdentifier('id'), typescript_1.factory.createIdentifier('diffCount')])),
                        ], true), undefined),
                    ], true), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToUnlinkMap), typescript_1.factory.createIdentifier('set')), undefined, [typescript_1.factory.createIdentifier('id'), typescript_1.factory.createIdentifier('count')])),
                    ], true)),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(updatedMap), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('count'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('id'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('originalCount'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(originalMap), typescript_1.factory.createIdentifier('get')), undefined, [typescript_1.factory.createIdentifier('id')])),
                    ], typescript_1.NodeFlags.Const)),
                    typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('originalCount'), typescript_1.factory.createBlock([
                        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('diffCount'), undefined, undefined, typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('count'), typescript_1.factory.createToken(typescript_1.SyntaxKind.MinusToken), typescript_1.factory.createIdentifier('originalCount'))),
                        ], typescript_1.NodeFlags.Const)),
                        typescript_1.factory.createIfStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('diffCount'), typescript_1.factory.createToken(typescript_1.SyntaxKind.GreaterThanToken), typescript_1.factory.createNumericLiteral('0')), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLinkMap), typescript_1.factory.createIdentifier('set')), undefined, [typescript_1.factory.createIdentifier('id'), typescript_1.factory.createIdentifier('diffCount')])),
                        ], true), undefined),
                    ], true), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLinkMap), typescript_1.factory.createIdentifier('set')), undefined, [typescript_1.factory.createIdentifier('id'), typescript_1.factory.createIdentifier('count')])),
                    ], true)),
                ], true)),
            ])),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('promises'), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([], false)),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToUnlinkMap), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('count'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('id'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier((0, form_state_1.getRecordsName)(relatedJoinTableName)), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('query')), undefined, [
                            typescript_1.factory.createIdentifier(relatedJoinTableName),
                            typescript_1.factory.createArrowFunction(undefined, undefined, [
                                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('and')), undefined, [
                                typescript_1.factory.createArrowFunction(undefined, undefined, [
                                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createArrayLiteralExpression([
                                    typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier(relatedJoinFieldName)), typescript_1.factory.createIdentifier('id')), typescript_1.factory.createIdentifier('eq')), undefined, [typescript_1.factory.createIdentifier('id')]),
                                    typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier(relatedModelField)), typescript_1.factory.createIdentifier('id')), typescript_1.factory.createIdentifier('eq')), undefined, [
                                        typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`), typescript_1.factory.createIdentifier('id')),
                                    ]),
                                ], false)),
                            ])),
                        ]))),
                    ], 
                    // eslint-disable-next-line no-bitwise
                    typescript_1.NodeFlags.Const | typescript_1.NodeFlags.AwaitContext | typescript_1.NodeFlags.ContextFlags | typescript_1.NodeFlags.TypeExcludesFlags)),
                    typescript_1.factory.createForStatement(typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('i'), undefined, undefined, typescript_1.factory.createNumericLiteral('0')),
                    ], 
                    // eslint-disable-next-line no-bitwise
                    typescript_1.NodeFlags.Let | typescript_1.NodeFlags.AwaitContext | typescript_1.NodeFlags.ContextFlags | typescript_1.NodeFlags.TypeExcludesFlags), typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('i'), typescript_1.factory.createToken(typescript_1.SyntaxKind.LessThanToken), typescript_1.factory.createIdentifier('count')), typescript_1.factory.createPostfixUnaryExpression(typescript_1.factory.createIdentifier('i'), typescript_1.SyntaxKind.PlusPlusToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('delete')), undefined, [
                                typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier((0, form_state_1.getRecordsName)(relatedJoinTableName)), typescript_1.factory.createIdentifier('i')),
                            ]),
                        ])),
                    ], true)),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLinkMap), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('count'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('id'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createForStatement(typescript_1.factory.createVariableDeclarationList([
                        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('i'), undefined, undefined, typescript_1.factory.createIdentifier('count')),
                    ], typescript_1.NodeFlags.Let), typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('i'), typescript_1.factory.createToken(typescript_1.SyntaxKind.GreaterThanToken), typescript_1.factory.createNumericLiteral('0')), typescript_1.factory.createPostfixUnaryExpression(typescript_1.factory.createIdentifier('i'), typescript_1.SyntaxKind.MinusMinusToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                                typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(relatedJoinTableName), undefined, [
                                    typescript_1.factory.createObjectLiteralExpression([
                                        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(`${relatedModelField}ID`), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`), typescript_1.factory.createIdentifier('id'))),
                                        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(`${relatedJoinFieldName}ID`), typescript_1.factory.createIdentifier('id')),
                                    ], true),
                                ]),
                            ]),
                        ])),
                    ], true)),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                    typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                        typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`),
                        typescript_1.factory.createArrowFunction(undefined, undefined, [
                            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('assign')), undefined, [typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier('modelFields')])),
                        ], true)),
                    ]),
                ]),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [typescript_1.factory.createIdentifier('promises')]))),
        ];
    }
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(relatedModelField), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(modelName), undefined, [
                    typescript_1.factory.createIdentifier('modelFields'),
                ]),
            ]))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('reduce')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(relatedJoinFieldName), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                            typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(relatedJoinTableName), undefined, [
                                typescript_1.factory.createObjectLiteralExpression([
                                    typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(relatedModelField), undefined),
                                    typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(relatedJoinFieldName), undefined),
                                ], true),
                            ]),
                        ]),
                    ])),
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                ], true)),
                typescript_1.factory.createArrayLiteralExpression([], false),
            ]),
        ]))),
    ];
};
exports.buildManyToManyRelationshipDataStoreStatements = buildManyToManyRelationshipDataStoreStatements;
const buildGetRelationshipModels = (fieldName) => {
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(`${fieldName}Record`), undefined, undefined, typescript_1.factory.createConditionalExpression(typescript_1.factory.createIdentifier('record'), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createAwaitExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('record'), typescript_1.factory.createIdentifier(fieldName))), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier('undefined'))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression((0, form_state_1.getSetNameIdentifier)(fieldName), undefined, [
            typescript_1.factory.createIdentifier(`${fieldName}Record`),
        ])),
    ];
};
exports.buildGetRelationshipModels = buildGetRelationshipModels;
const buildHasManyRelationshipDataStoreStatements = (dataStoreActionType, modelName, hasManyFieldConfig) => {
    const [fieldName, fieldConfig] = hasManyFieldConfig;
    const { relatedModelName, relatedModelField } = fieldConfig.relationship;
    const linkedDataName = (0, form_state_1.getLinkedDataName)(fieldName);
    const dataToLink = `${(0, helpers_1.lowerCaseFirst)(fieldName)}ToLink`;
    const dataToUnLink = `${(0, helpers_1.lowerCaseFirst)(fieldName)}ToUnLink`;
    const dataToLinkSet = `${(0, helpers_1.lowerCaseFirst)(fieldName)}Set`;
    const linkedDataSet = `${linkedDataName}Set`;
    if (dataStoreActionType === 'update') {
        return [
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(dataToLink), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([], false)),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(dataToUnLink), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([], false)),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(dataToLinkSet), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Set'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(linkedDataSet), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Set'), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLinkSet), typescript_1.factory.createIdentifier('add')), undefined, [typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id'))])),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(linkedDataName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(linkedDataSet), typescript_1.factory.createIdentifier('add')), undefined, [typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id'))])),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(linkedDataName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createIfStatement(typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLinkSet), typescript_1.factory.createIdentifier('has')), undefined, [
                        typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                    ])), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToUnLink), typescript_1.factory.createIdentifier('push')), undefined, [typescript_1.factory.createIdentifier('r')])),
                    ], true), undefined),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createIfStatement(typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(linkedDataSet), typescript_1.factory.createIdentifier('has')), undefined, [
                        typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('id')),
                    ])), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLink), typescript_1.factory.createIdentifier('push')), undefined, [typescript_1.factory.createIdentifier('r')])),
                    ], true), undefined),
                ], true)),
            ])),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('promises'), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([], false)),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToUnLink), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('original'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(relatedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                                typescript_1.factory.createIdentifier('original'),
                                typescript_1.factory.createArrowFunction(undefined, undefined, [
                                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier(relatedModelField)), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createNull())),
                                ], true)),
                            ]),
                        ]),
                    ])),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(dataToLink), typescript_1.factory.createIdentifier('forEach')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('original'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(relatedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                                typescript_1.factory.createIdentifier('original'),
                                typescript_1.factory.createArrowFunction(undefined, undefined, [
                                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier(relatedModelField)), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`), typescript_1.factory.createIdentifier('id')))),
                                ], true)),
                            ]),
                        ]),
                    ])),
                ], true)),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                    typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                        typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`),
                        typescript_1.factory.createArrowFunction(undefined, undefined, [
                            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('assign')), undefined, [typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier('modelFields')])),
                        ], true)),
                    ]),
                ]),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [typescript_1.factory.createIdentifier('promises')]))),
        ];
    }
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(modelName.toLowerCase()), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(modelName), undefined, [
                    typescript_1.factory.createIdentifier('modelFields'),
                ]),
            ]))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('reduce')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('original'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(relatedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                                typescript_1.factory.createIdentifier('original'),
                                typescript_1.factory.createArrowFunction(undefined, undefined, [
                                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier(relatedModelField)), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(modelName.toLowerCase()), typescript_1.factory.createIdentifier('id')))),
                                ], true)),
                            ]),
                        ]),
                    ])),
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                ], true)),
                typescript_1.factory.createArrayLiteralExpression([], false),
            ]),
        ]))),
    ];
};
exports.buildHasManyRelationshipDataStoreStatements = buildHasManyRelationshipDataStoreStatements;
//# sourceMappingURL=relationship.js.map