"use strict";
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.resetFunctionCheck = exports.renderValueAttribute = exports.renderDefaultValueAttribute = exports.convertedValueAttributeMap = exports.isControlledComponent = exports.ControlledComponents = void 0;
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
const render_checkers_1 = require("./render-checkers");
exports.ControlledComponents = ['StepperField', 'SliderField', 'SelectField', 'ToggleButton', 'SwitchField'];
/**
 * given the component returns true if the component is a controlled component
 *
 * @param componentType
 * @returns
 */
const isControlledComponent = (componentType) => exports.ControlledComponents.includes(componentType);
exports.isControlledComponent = isControlledComponent;
exports.convertedValueAttributeMap = {
    AWSTimestamp: (valueIdentifier) => typescript_1.factory.createBinaryExpression(valueIdentifier, typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('convertToLocal'), undefined, [
        typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('convertTimeStampToDate'), undefined, [valueIdentifier]),
    ])),
    AWSDateTime: (valueIdentifier) => typescript_1.factory.createBinaryExpression(valueIdentifier, typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('convertToLocal'), undefined, [
        typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Date'), undefined, [valueIdentifier]),
    ])),
};
/**
 *
 * @param stateName name of the state name to render for the default value
 * @param { dataType } the dataType
 * @returns
 */
const renderDefaultValueAttribute = (stateName, { dataType }) => {
    const identifier = typescript_1.factory.createIdentifier(stateName);
    let expression = typescript_1.factory.createJsxExpression(undefined, identifier);
    if (dataType && typeof dataType !== 'object' && exports.convertedValueAttributeMap[dataType]) {
        expression = typescript_1.factory.createJsxExpression(undefined, exports.convertedValueAttributeMap[dataType](identifier));
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('defaultValue'), expression);
};
exports.renderDefaultValueAttribute = renderDefaultValueAttribute;
const renderValueAttribute = ({ fieldConfig, componentName, }) => {
    var _a;
    const componentType = (_a = fieldConfig.studioFormComponentType) !== null && _a !== void 0 ? _a : fieldConfig.componentType;
    const shouldGetForUncontrolled = (0, render_checkers_1.shouldWrapInArrayField)(fieldConfig);
    let valueIdentifier = typescript_1.factory.createIdentifier(componentName.split('.')[0]);
    if ((0, render_checkers_1.shouldWrapInArrayField)(fieldConfig)) {
        if ((0, render_checkers_1.isModelDataType)(fieldConfig)) {
            valueIdentifier = typescript_1.factory.createIdentifier((0, form_state_1.getCurrentDisplayValueName)(componentName));
        }
        else {
            valueIdentifier = typescript_1.factory.createIdentifier((0, form_state_1.getCurrentValueName)(componentName));
        }
    }
    const controlledComponentToAttributesMap = {
        ToggleButton: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('isPressed'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
        SliderField: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('value'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
        SelectField: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('value'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
        StepperField: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('value'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
        SwitchField: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('isChecked'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
        CheckboxField: typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('checked'), typescript_1.factory.createJsxExpression(undefined, valueIdentifier)),
    };
    if (controlledComponentToAttributesMap[componentType]) {
        return controlledComponentToAttributesMap[componentType];
    }
    // TODO: all components should be controlled once conversions are solid
    if (shouldGetForUncontrolled) {
        const { dataType } = fieldConfig;
        let expression = typescript_1.factory.createJsxExpression(undefined, valueIdentifier);
        if (dataType && typeof dataType !== 'object' && exports.convertedValueAttributeMap[dataType]) {
            expression = typescript_1.factory.createJsxExpression(undefined, exports.convertedValueAttributeMap[dataType](valueIdentifier));
        }
        return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('value'), expression);
    }
    return undefined;
};
exports.renderValueAttribute = renderValueAttribute;
/**
 * returns resetFunctionCheck Expression if a create form is datastore enabled
 *
 * @param studioForm {StudioForm}
 * @returns
 */
const resetFunctionCheck = ({ dataSourceType, formActionType, }) => {
    if (dataSourceType === 'DataStore' && formActionType === 'create') {
        return [
            typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('clearOnSuccess'), typescript_1.factory.createBlock([typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(form_state_1.resetValuesName, undefined, []))], true), undefined),
        ];
    }
    return [];
};
exports.resetFunctionCheck = resetFunctionCheck;
//# sourceMappingURL=value-props.js.map